'''Some simple geometric classes.'''

import math



class Position:
    def __init__(self, x, y):
        self.x = x
        self.y = y
    
    
    def in_direction(self, dir):
        return Position(self.x + dir.dx, self.y + dir.dy)
    
    def translate(self, dir):
        self.x += dir.dx
        self.y += dir.dy
    
    def position(self):
        return Position(self.x, self.y)
    
    def direction(self):
        return Direction(self.x, self.y)
    
    def direction_to(self, other):
        return Direction(other.x - self.x, other.y - self.y)
    
    def distance(self, other, half_x = False):
        if half_x:
            return math.sqrt(((self.x - other.x) / 2) ** 2 + 
                             (self.y - other.y) ** 2)
            
        return math.sqrt((self.x - other.x) ** 2 + (self.y - other.y) ** 2)
    
    def __repr__(self):
        return 'Position(%d, %d)' % (self.x, self.y)



class Rectangle:
    def __init__(self, x, y, width, height):
        self.x = x
        self.y = y
        self.width = width
        self.height = height
    
    def area(self):
        return self.width * self.height
    
    def diagonal(self):
        return Direction(self.width, self.height)
    
    def __repr__(self):
        return 'Rectangle(%d, %d, %d, %d)' % (self.x, self.y, 
                                              self.width, self.height)
        
        

class Direction:
    def __init__(self, dx, dy, name = None):
        self.dx = dx
        self.dy = dy
        self.name = name
    
    def invert(self):
        self.dx *= -1
        self.dy *= -1
    
    def scale(self, factor):
        self.dx *= factor
        self.dy *= factor
    
    def descale(self, divisor):
        self.dx /= divisor
        self.dy /= divisor
    
    def compass_name(self):
        x_name = None
        y_name = None
        
        if self.dx < 0:
            x_name = 'west'
        
        if self.dx > 0:
            x_name = 'east'
        
        if self.dy < 0:
            y_name = 'north'
        
        if self.dy > 0:
            y_name = 'south'
            
        if x_name is None:
            return y_name
            
        if y_name is None:
            return x_name
            
        return '%s-%s' % (y_name, x_name)
        
        
        
    def __str__(self):
        if self.name is None:
            return 'd(%d, %d)' % (self.dx, self.dy)
        
        return self.name



COMPASS = {'North': Direction( 0, -1, 'North'),
           'South': Direction( 0,  1, 'South'),
           'West' : Direction(-1,  0, 'West'),
           'East' : Direction( 1,  0, 'East')}
