'''Moving units.'''

import random

from concepts.geometry import Position



class Unit(Position):
    def __init__(self, x, y, map, name):
        Position.__init__(self, x, y)
        
        self.name = name
        self.map = map
    
    def can_move(self, dir):
        tile = self.map.tile(self.in_direction(dir))
        
        return (tile is not None and tile.can_move_in(self))
    
    def move(self, dir):
        if self.can_move(dir):
            tile = self.map.tile(self.in_direction(dir))
            self.translate(dir)
            tile.move_in(self)
            
    def __str__(self):
        return self.name



class Avatar(Unit):
    def __init__(self, model, x, y, map):
        Unit.__init__(self, x, y, map, 'Avatar')
        
        self._model  = model
        self._events = []
        
    def model(self):
        return self._model
    
    def current_tile(self):
        return self.map.tile(self)
    
    def events(self):
        return self._events
    
    def do_events(self):
        self._events = self.model().new_events()
        
        if not self._events:
            self._events = self.current_tile().events(self)

        for event in self._events:
            event.do()
        
        self.do_self_events()
        
    def do_self_events(self):
        self_events   = self.collect_self_events()
        self._events += self_events
        
        for event in self_events:
            event.do()
    
    def collect_self_events(self):
        return []
    
    def situation(self):
        raise 'Not implemented'
    


class Gender:
    def __init__(self, name, nom, acc, pos_adj, pos_pro, refl):
        self._name    = name
        self._nom     = nom
        self._acc     = acc
        self._pos_adj = pos_adj
        self._pos_pro = pos_pro
        self._refl    = refl
        
    def name(self):
        return self._name
    
    def nominative(self):
        return self._nom
    
    def accusative(self):
        return self._acc
    
    def possessive_adjective(self):
        return self._pos_adj
    
    def possessive_pronoun(self):
        return self._pos_pro
    
    def reflexive(self):
        return self._refl
    
    def __str__(self):
        return self.name()



def MALE():
    return Gender('male', 'he', 'him', 'his', 'his', 'himself')

def FEMALE():
    return Gender('female', 'she', 'her', 'her', 'hers', 'herself')

def THIRD_GENDER():
    return Gender('third-gender', 'sie', 'hir', 'hir', 'hirs', 'hirself')

def RANDOM_GENDER(m = 49, f = 49, t = 2):
    chance = random.randint(1, m + f + t)
    
    if chance <= m:
        return MALE()
    
    if chance <= m + f:
        return FEMALE()
    
    return THIRD_GENDER()
