'''Wrapper around curses.'''

from ui import cursesemu

curses = cursesemu



class CursesUI:
    def main(self):
        cursesemu.wrapper(lambda stdscr: self.set_up_and_run(stdscr))
        
    def set_up_and_run(self, stdscr):
        cursesemu.curs_set(0)
        
        self.stdscr = stdscr
        self.init()
        self.run()
    
    def get_key(self):
        return self.stdscr.getch()
        
    def init(self):
        raise 'Not implemented'
        
    def run(self):
        raise 'Not implemented'



class CursesGameUI(CursesUI):    
    def __init__(self, model):
        self.model = model    
        self._full = False
        
    def init(self):
        pass
    
    def full(self):
        return self._full
    
    def run(self):
        self.do_game()
            
    def do_game(self):
        self.model.start()
        
        while self.model.is_running():
            self.model.do_turn()
            self.do_turn()
        
        self.do_scores()
        
    def prepare_turn(self):
        pass
    
    def do_turn(self):
        cursesemu.flushinp()
        self.prepare_turn()
        self.display()
        self.act()
    
    def do_scores(self):
        pass
    
    def display(self):
        if self.full():
            self.display_full()
        else:
            for window in self.displayed_windows():
                window.display()
                window.sync()
            
        self.stdscr.refresh()
    
    def display_full(self):
        pass
    
    def create_window(self, kind, rect, border = True):
        return kind(self, self.stdscr, self.model, rect, border)
    
    def act(self):
        acted = False
        
        while not acted:
            key = self.get_key()
            actions = self.model.available_actions()
            
            for action in actions:
                if self.key_for(action) == key:
                    acted = action.execute()
                        
    def char_for(self, action):
        raise 'Not implemented'
    
    def key_for(self, action):
        raise 'Not implemented'
    
    def group_for(self, action):
        raise 'Not implemented'
        
    def displayed_windows(self):
        raise 'Not implemented'
    
    def window(self, name):
        raise 'Not implemented'
    